#ifndef _MASYRO
#define _MASYRO

#include <FIPA.ice>

module FIPA
{

struct TZone {    // Define una zona a renderizar.
	int id;   // Identificador de la zona
	int x1;   // Coordenadas que definen la zona
	int y1;
	int x2;
	int y2;
	float d;  // Desviacin estandar del color de la zona
	float m;  // Media del color en la zona
};

sequence <TZone> TZones;
enum StateRegister {Done, NotDone, InWork};

struct TRegister {    // Registro de la pizarra
	int IdWork;     	// Identificador del trabajo
	int WorkUnit;   	// Identificador de la unidad de trabajo
	int Size;       	// Tamano (pixels) de la unidad de trabajo
	int Comp;       	// Complejidad de la Unidad de Trabajo
	int Test; 		// Tiempo estimado (Path tracing)	
	int TestRT;     	// Tiempo estimado (Ray tracing)
	int TestRC;     	// Tiempo estimado (Ray casting)
	float QDRT;     	// Quality Difference Ray Tracing
	float QDRC;     	// Quality Difference Ray Casting
	string Method;  	// Render Method (si se usan varios)
	int Treal;      	// Tiempo final empleado por el agente.
	string Agent;   	// Nombre del agente encargado de la unidad de trabajo.
	StateRegister State;    // Se completo la unidad de trabajo?
	int Ibs;        	// Interpolation Band Size
	int Ls;         	// Light Samples
	int Rl;         	// Recursion Level
};

sequence <TRegister> TRegisters;

sequence <byte> ByteSeq;
sequence <int> IntSeq;
sequence <string> StringSeq;

enum StateRenderAgent {Estimating, Bidding, Rendering, Resting, Finishing};

// Defines the internal state of the Agent
struct TAgentState {
	string Name;	// Name of the Agent
        IntSeq CurrentZones;
	int CurrentIdWork;
	int Credits;
        IntSeq Historic;
        float Coefficient;  // Internal Coefficient of the Agent
        int BenchmarkValue;
	StateRenderAgent State;

};

/***************************/
/*Definicin de excepciones*/
/***************************/

// Excepcin base.
exception BaseException
{
	string Reason;
};

// Excepcin lanzada en caso de que al solicitar un modelo ste no exista.
exception ModelNotExistsException extends BaseException
{
	IntSeq ExistingModels;
};

// Excepcin lanzada en caso de que no exista un registro a la hora de actualizarlo.
exception RegisterNotExistsException extends BaseException
{
};

/***************************/
/*Descripcin de interfaces.*/
/***************************/

// Analyst es el servicio encargado de llevar a cabo el anlisis de la entrada.
interface Analyst extends Agent
{

	// La operacin processWork permite procesar un trabajo asociado a una escena.
	// work representa el flujo de bytes asociado al trabajo.
	// workName es el nombre dado al trabajo.
	// level representa el nmero de pasadas para llevar a cabo la divisin de la imagen inicial en trozos.
	// optimization es el nivel de optimizacin definido por el usuario.
	// rendermethods: si es 1, solo se usara un metodo de render, si es 0 se usaran varios
	// Devuelve: int -> el identificador del trabajo
 	// int processWork(ByteSeq work, string workName, int level, int optimization);
	
	int processWork(string work, string workName, int level, int optimization, int rendermethods);
        
        // Notifica un trabajo nuevo al Master.
        // Id: identificador del trabajo devuelto en processWork.
        // Optimization: Nivel de optimizacion.
	void notifyNewWorkToMaster(int id, int optimization, int rendermethods);

	// getZonesByPartition: Dado un nivel de particin, devuelve la lista de zonas (particion).
	idempotent TZones getZonesByPartition (int partition);	

	// getFileInfo: Return a String with file information (encoded with ":")
	idempotent string getFileInfo();

};

// RenderAgent es la interfaz a un agente especializado en el render.
interface RenderAgent extends Agent
{

	// La operacin notifyNewWork permite al RenderAgent conocer la existencia de un nuevo trabajo.
	void notifyNewWork(TZones zones, int idWork, int benchmarkValue, int rendermethods);

	// NotifyZone asigna una lista de zonas del nuevo trabajo al RenderAgent, para que ste haga un estudio previo.
	// optimization ==> Nivel de optimizacin definido por el usuario (1-5).
	["ami"] void notifyZones(TZones zones, int idWork, int optimization);

	// La operacin beginRenderProcess notifica el comienzo del trabajo.
	void beginRenderProcess();

	// La operacin render notifica al agente el comienzo del renderizado para la zona idZone del trabajo idWork.
	// method:  PT (PathTracing)  RT (RayTracing)  RC (RayCasting)
	["ami"] void render(int idWork, int idZone, string agent, string method);

	// La operacin getRenderState permite obtener el estado del agente.
	idempotent StateRenderAgent getRenderState();

	// getInternalState: Obtain the internal state of an Agent
	idempotent TAgentState getInternalState(); 
};

// Master es el servicio publicador de la existencia de nuevos trabajos en la plataforma.
interface Master extends Agent
{

	// La operacin subscribe permite a un agente especializado en el renderizado subscribirse al Master.
	void subscribe(string agentName, RenderAgent* agent);
	// La operacin unsubscribe permite a un agente especializado en el renderizado darse de baja con el Master.
	void unsubscribe(string agentName);
	// La operacin clear limpia el registro de agentes existentes y termina sus ejecuciones.
	void clean();

	// La operacin notifyNewWork permite al Master conocer la existencia de un nuevo trabajo.	
	// optimization ==> Nivel de optimizacin definido por el usuario (1-5).
	void notifyNewWork(TZones zones, int idWork, int optimization, int rendermethods);
	// La operacin benchmarkValue incrementa el valor medio del tiempo empleado en la ejecucin del benchmark.
	void benchmarkValue(int value);

	// La operacin bidHigher permite a un agente pujar por un trozo.
	["ami"] void bidHigher(string agent, int idWork, int idZone, int credits, IntSeq historic);
	// La operacin noMoreBiddings le indica al Master la posible finalizacin del trabajo.
	void noMoreBiddings();
	// La operacin giveFinalImage le proporciona al Master un trozo de la imagen final.
	// Cut: Si vale 1 (true), hay que recortarla. Si vale (0) ya viene recortada
	void giveFinalImage(int idWork, int idZone, ByteSeq partialImage, int x1, int y1, int x2, int y2, int ibs);

	// La operacin showAgentsState devuelve el estado de todos los agentes.
	idempotent string showAgentStates();
	// La operacin getLog devuelve el log del Master.
	idempotent string getLog();
	// La operacin getFinalTimes devuelve los tiempos finales empleados en un trabajo.
	idempotent string getFinalTimes();
	// Devuelve el numero de agentes activos.
	idempotent int getNumberofAgents();

	// getBlackboardState devuelve el estado actual de la pizarra.
	idempotent TRegisters getBlackboardState(); 

	// getInternalStateOfAgent(string agentName)
	idempotent TAgentState getInternalStateOfAgent(string agentName);

	// getAgentNames(): Return a list with the current connected agent names.
    	idempotent StringSeq getAgentNames();

};

// ModelRepository es la interfaz relativa al almacenamiento/recuperacin de modelos.
interface ModelRepository extends Agent
{
	// put devuelve el identificador asignado al modelo enviado.
	int put(string name, ByteSeq model);
	// get devuelve el nombre del modelo y el propio modelo como secuencia de bytes.
	idempotent string get(int idModel, out ByteSeq model) throws ModelNotExistsException;
	// put devuelve el identificador asignado al modelo enviado.
	void putWU(string name, ByteSeq model);
	// get devuelve el nombre del modelo y el propio modelo como secuencia de bytes.
	idempotent void getWU(string name, out ByteSeq model) throws ModelNotExistsException;

};

// Blackboard es la interfaz relativa a la arquitectura de pizarra.
interface Blackboard extends Agent
{

	// write escribe un registro en la pizarra.
	void write(TRegister register);
	// read lee un registro de la pizarra, identificado por el id del trabajo y el id de la zona.
	TRegister read(int idWork, int workUnit) throws RegisterNotExistsException;
	// update actualiza el valor de un registro en la pizarra.
	void update(int idWork, int workUnit, int test) throws RegisterNotExistsException;
	// update actualiza el valor de un registro en la pizarra.
	void updateRT(int idWork, int workUnit, int test) throws RegisterNotExistsException;
	// update actualiza el valor de un registro en la pizarra.
	void updateRC(int idWork, int workUnit, int test) throws RegisterNotExistsException;
	// update actualiza el valor de un registro en la pizarra.
	void updatePercRT(int idWork, int workUnit, float perc) throws RegisterNotExistsException;
	// update actualiza el valor de un registro en la pizarra.
	void updatePercRC(int idWork, int workUnit, float perc) throws RegisterNotExistsException;
	// update actualiza el valor de un registro en la pizarra.
	void updateMethod(int idWork, int workUnit, string method) throws RegisterNotExistsException;
	// clear limpia la pizarra.
	void clear();

	// Operaciones para la gestion del tiempo
	///////////////////////////////////////////////////////////////////////////////////////
	idempotent int stGetCurrentTime();   // Devuelve el tiempo actual.
	idempotent IntSeq stGetTimes();	     // Devuelve un array con todos los marcadores
	void stResetTimes();		     // Resetea los marcadores de tiempo (a cero).
	void stSetInitTime();                // Marca el tiempo de inicio.
	void stSetUploadTime();		     // Marca el tiempo de fin del upload.
	void stSetAnalysisTime();	     // Marca el tiempo de fin del Analisis.
	void stSetEstimationTime();	     // Marca el tiempo de fin de la Estimacin.
	void stSetBiddingTime(int time);     // Marca el tiempo de Bidding (total!).
	void stSetRenderTime();		     // Marca el tiempo de final del Render.
	///////////////////////////////////////////////////////////////////////////////////////	

	// setEstimationTime establece el tiempo empleado para realizar la estimacin.
	void setEstimationTime(int time);
	// getEstimationTime devuelve el tiempo empleado para realizar la estimacin.
	idempotent int getEstimationTime();

	// setAnalysisTime establece el tiempo empleado para el anlisis de la escena ms reciente.
	void setAnalysisTime(int time);
	// getAnalysisTime devuelve el tiempo empleado para el anlisis de la escena ms reciente.
	idempotent int getAnalysisTime();
	// incrementEstimatedRenderTime incrementa el tiempo de estimacin de la escena.
	void incrementEstimatedRenderTime(int time);
	// getEstimatedRenderTime devuelve el tiempo de estimacin de la escena.
	idempotent int getEstimatedRenderTime();

	// isWorkEstimated indica si las distintas partes del trabajo se han estimado.
	bool isWorkPartiallyEstimated();
	// isEnd indica si el trabajo actual ha terminado.
	bool isCurrentWorkFinished();
	// show muestra el contenido de la pizarra.
	string show();
	// getMaxTest devuelve el valor del trozo cuyo tiempo estimado es el mayor del trabajo idWork.
	int getMaxTest(int idWork);
	// getMaxComp devuelve el valor del trozo cuya complejidad es la mayor del trabajo idWork
	int getMaxComp(int idWork);
	// getTimeCurrentWorks devuelve el tiempo empleado en el trabajo.
	idempotent string getTimeCurrentWorks();

	// setWorkUnit permite a un agente hacerse cargo de una unidad de trabajo.
	void setWorkUnit(int idWork, int workUnit, string agent) throws RegisterNotExistsException;
	// finishWork permite a un agente notificar la finalizacin de una unidad de trabajo.
	void finishWorkUnit(int idWork, int workUnit, int treal, int ibs, int ls, int rl) throws RegisterNotExistsException;

};

};

#endif
